import { EntityType } from '../types/entity.type';
import { BaseEntity } from '../interfaces/base-entity.interface';
import { IEntityOptions } from '../decorators/entity.decorator';
import { IRelationOptions } from '../decorators/relation.decorator';
import { IResourceIdentifierObject, IResourceObject } from '../interfaces/response';

export class ResourceSerializer<T extends BaseEntity> {
  constructor(private type: EntityType<T>) {}

  public handleOne(data: T): IResourceObject<T> | IResourceIdentifierObject<T> | null {
    const { id, ...attributes } = data;

    const options: IEntityOptions = Reflect.getMetadata('SERIALIZER_ENTITY', this.type);
    if (!options) {
      throw new Error(`Could not resolve entity type for class ${ JSON.stringify(this.type) }. Are you missing an @Entity decorator?`);
    }

    const relations: { [key: string]: IRelationOptions } = Reflect.getMetadata('SERIALIZER_RELATION', this.type) || {};
    Object.keys(relations).forEach((key) => {
      delete (<any>attributes)[key];
    });
    
    return { id, type: options.name, attributes };
  }
  
  public handleMany(data: T[]): IResourceObject<T>[] | IResourceIdentifierObject<T>[] | [] {
    return data.map((item: T) => {
      const { id, ...attributes } = item;

      const options: IEntityOptions = Reflect.getMetadata('SERIALIZER_ENTITY', this.type);
      if (!options) {
        throw new Error(`Could not resolve entity type for class ${ JSON.stringify(this.type) }. Are you missing an @Entity decorator?`);
      }

      const relations: { [key: string]: IRelationOptions } = Reflect.getMetadata('SERIALIZER_RELATION', this.type) || {};
      Object.keys(relations).forEach((key) => {
        delete (<any>attributes)[key];
      });
      
      return { id, type: options.name, attributes };
    });
  }
}

// const relations = Reflect.getMetadata('SERIALIZER_RELATION', this.entity) || {};